import { jsx as _jsx, jsxs as _jsxs } from "@emotion/react/jsx-runtime";
import { describe, jest, it, expect } from '@jest/globals';
import { fireEvent, render, screen } from '@testing-library/react';
import userEvent from '@testing-library/user-event';
import { ContextMenu } from './ContextMenu';
import { DesignSystemEventProvider } from '../DesignSystemEventProvider';
describe('ContextMenu', () => {
    const eventCallback = jest.fn();
    const CommonComponent = ({ children }) => (_jsx(DesignSystemEventProvider, { callback: eventCallback, children: _jsxs(ContextMenu.Root, { children: [_jsx(ContextMenu.Trigger, { children: "Trigger" }), _jsx(ContextMenu.Content, { children: children })] }) }));
    it('emits click event for item', async () => {
        const onClickSpy = jest.fn();
        render(_jsx(CommonComponent, { children: _jsx(ContextMenu.Item, { componentId: "context_menu_item_test", onClick: onClickSpy, children: "Item" }) }));
        expect(onClickSpy).not.toHaveBeenCalled();
        expect(eventCallback).not.toHaveBeenCalled();
        fireEvent.contextMenu(screen.getByText('Trigger'));
        await userEvent.click(screen.getByRole('menuitem', { name: 'Item' }));
        expect(onClickSpy).toHaveBeenCalledTimes(1);
        expect(eventCallback).toHaveBeenCalledTimes(1);
        expect(eventCallback).toHaveBeenCalledWith({
            eventType: 'onClick',
            componentId: 'context_menu_item_test',
            componentType: 'context_menu_item',
            shouldStartInteraction: true,
            value: undefined,
            event: expect.any(Object),
            isInteractionSubject: undefined,
        });
    });
    it('does not emit click event for item when asChild is set', async () => {
        const onClickSpy = jest.fn();
        render(_jsx(CommonComponent, { children: _jsx(ContextMenu.Item, { componentId: "context_menu_item_test", onClick: onClickSpy, asChild: true, children: _jsx("button", { children: "Item" }) }) }));
        expect(onClickSpy).not.toHaveBeenCalled();
        expect(eventCallback).not.toHaveBeenCalled();
        fireEvent.contextMenu(screen.getByText('Trigger'));
        await userEvent.click(screen.getByRole('menuitem', { name: 'Item' }));
        expect(onClickSpy).toHaveBeenCalledTimes(1);
        expect(eventCallback).not.toHaveBeenCalled();
    });
    it('emits value change event for checkbox item', async () => {
        const onCheckedChangeSpy = jest.fn();
        render(_jsx(CommonComponent, { children: _jsx(ContextMenu.CheckboxItem, { componentId: "context_menu_checkbox_item_test", onCheckedChange: onCheckedChangeSpy, children: "Checkbox Item" }) }));
        expect(onCheckedChangeSpy).not.toHaveBeenCalled();
        expect(eventCallback).not.toHaveBeenCalled();
        fireEvent.contextMenu(screen.getByText('Trigger'));
        await userEvent.click(screen.getByRole('menuitemcheckbox', { name: 'Checkbox Item' }));
        expect(onCheckedChangeSpy).toHaveBeenCalledWith(true);
        expect(eventCallback).toHaveBeenCalledTimes(1);
        expect(eventCallback).toHaveBeenCalledWith({
            eventType: 'onValueChange',
            componentId: 'context_menu_checkbox_item_test',
            componentType: 'context_menu_checkbox_item',
            shouldStartInteraction: false,
            value: true,
        });
    });
    it('emits value change for radio group', async () => {
        const onValueChangeSpy = jest.fn();
        render(_jsx(CommonComponent, { children: _jsxs(ContextMenu.RadioGroup, { componentId: "context_menu_radio_group_test", valueHasNoPii: true, onValueChange: onValueChangeSpy, children: [_jsx(ContextMenu.RadioItem, { value: "one", children: "Radio Item 1" }), _jsx(ContextMenu.RadioItem, { value: "two", children: "Radio Item 2" })] }) }));
        expect(onValueChangeSpy).not.toHaveBeenCalled();
        expect(eventCallback).not.toHaveBeenCalled();
        fireEvent.contextMenu(screen.getByText('Trigger'));
        await userEvent.click(screen.getByRole('menuitemradio', { name: 'Radio Item 1' }));
        expect(onValueChangeSpy).toHaveBeenCalledWith('one');
        expect(eventCallback).toHaveBeenCalledTimes(1);
        expect(eventCallback).toHaveBeenCalledWith({
            eventType: 'onValueChange',
            componentId: 'context_menu_radio_group_test',
            componentType: 'context_menu_radio_group',
            shouldStartInteraction: false,
            value: 'one',
        });
        fireEvent.contextMenu(screen.getByText('Trigger'));
        await userEvent.click(screen.getByRole('menuitemradio', { name: 'Radio Item 2' }));
        expect(onValueChangeSpy).toHaveBeenCalledWith('two');
        expect(eventCallback).toHaveBeenCalledTimes(2);
        expect(eventCallback).toHaveBeenCalledWith({
            eventType: 'onValueChange',
            componentId: 'context_menu_radio_group_test',
            componentType: 'context_menu_radio_group',
            shouldStartInteraction: false,
            value: 'two',
        });
    });
    it('emits value change event without value for radio group when valueHasNoPii is not set', async () => {
        const onValueChangeSpy = jest.fn();
        render(_jsx(CommonComponent, { children: _jsxs(ContextMenu.RadioGroup, { componentId: "context_menu_radio_group_test", onValueChange: onValueChangeSpy, children: [_jsx(ContextMenu.RadioItem, { value: "one", children: "Radio Item 1" }), _jsx(ContextMenu.RadioItem, { value: "two", children: "Radio Item 2" })] }) }));
        expect(onValueChangeSpy).not.toHaveBeenCalled();
        expect(eventCallback).not.toHaveBeenCalled();
        fireEvent.contextMenu(screen.getByText('Trigger'));
        await userEvent.click(screen.getByRole('menuitemradio', { name: 'Radio Item 1' }));
        expect(onValueChangeSpy).toHaveBeenCalledWith('one');
        expect(eventCallback).toHaveBeenCalledTimes(1);
        expect(eventCallback).toHaveBeenCalledWith({
            eventType: 'onValueChange',
            componentId: 'context_menu_radio_group_test',
            componentType: 'context_menu_radio_group',
            shouldStartInteraction: false,
            value: undefined,
        });
    });
});
//# sourceMappingURL=ContextMenu.test.js.map