import { describe, jest, it, expect, beforeEach } from '@jest/globals';
import { render, screen } from '@testing-library/react';
import userEvent from '@testing-library/user-event';
import React, { useState } from 'react';

import { Drawer } from '.';
import type { DrawerContentProps } from './Drawer';
import { DesignSystemEventProvider, DesignSystemEventProviderAnalyticsEventTypes } from '../DesignSystemEventProvider';
import { DesignSystemProvider } from '../DesignSystemProvider';
import { setupSafexTesting } from '../utils/safex';

describe('Drawer Analytics Events', () => {
  const { setSafex } = setupSafexTesting();
  const eventCallback = jest.fn();
  const Component = () => (
    <DesignSystemProvider>
      <DesignSystemEventProvider callback={eventCallback}>
        <Drawer.Root open={true}>
          <Drawer.Trigger>
            <button>Drawer Trigger</button>
          </Drawer.Trigger>
          <Drawer.Content
            title="drawer_title"
            componentId="drawer_test"
            analyticsEvents={[DesignSystemEventProviderAnalyticsEventTypes.OnView]}
          >
            <div>Main content goes here</div>
          </Drawer.Content>
        </Drawer.Root>
      </DesignSystemEventProvider>
    </DesignSystemProvider>
  );

  describe('disabled defaultButtonComponentView', () => {
    beforeEach(() => {
      setSafex({
        'databricks.fe.observability.defaultButtonComponentView': false,
      });
    });
    it('emits drawer content view close events', async () => {
      // Disable IntersectionObserver for useNotifyOnFirstView hook to trigger
      (window as any).IntersectionObserver = undefined;

      render(<Component />);

      expect(screen.getByText('Main content goes here')).toBeVisible();
      expect(eventCallback).toHaveBeenCalledTimes(1);
      expect(eventCallback).toHaveBeenCalledWith({
        componentId: 'drawer_test',
        componentType: 'drawer_content',
        eventType: 'onView',
        shouldStartInteraction: false,
        value: undefined,
      });

      const closeButton = screen.getByRole('button', { name: 'Close' });
      await userEvent.click(closeButton);

      expect(eventCallback).toHaveBeenCalledTimes(2);
      expect(eventCallback).toHaveBeenCalledWith({
        componentId: 'drawer_test.close',
        componentType: 'button',
        eventType: 'onClick',
        shouldStartInteraction: true,
        isInteractionSubject: true,
        event: expect.anything(),
      });
    });
  });

  describe('enabled defaultButtonComponentView', () => {
    beforeEach(() => {
      setSafex({
        'databricks.fe.observability.defaultButtonComponentView': true,
      });
    });
    it('emits drawer content view close events', async () => {
      // Disable IntersectionObserver for useNotifyOnFirstView hook to trigger
      (window as any).IntersectionObserver = undefined;

      render(<Component />);

      expect(screen.getByText('Main content goes here')).toBeVisible();
      expect(eventCallback).toHaveBeenCalledTimes(2);
      expect(eventCallback).toHaveBeenCalledWith({
        componentId: 'drawer_test.close',
        componentType: 'button',
        eventType: 'onView',
        shouldStartInteraction: false,
        value: undefined,
      });
      expect(eventCallback).toHaveBeenCalledWith({
        componentId: 'drawer_test',
        componentType: 'drawer_content',
        eventType: 'onView',
        shouldStartInteraction: false,
        value: undefined,
      });

      const closeButton = screen.getByRole('button', { name: 'Close' });
      await userEvent.click(closeButton);

      expect(eventCallback).toHaveBeenCalledTimes(3);
      expect(eventCallback).toHaveBeenNthCalledWith(3, {
        componentId: 'drawer_test.close',
        componentType: 'button',
        eventType: 'onClick',
        shouldStartInteraction: true,
        isInteractionSubject: true,
        event: expect.anything(),
      });
    });
  });
});

describe.each([false, true])(
  'Shared Drawer tests: databricks.fe.observability.defaultButtonComponentView set to %s',
  (defaultButtonComponentView: boolean) => {
    const { setSafex } = setupSafexTesting();
    beforeEach(() => {
      setSafex({
        'databricks.fe.observability.defaultButtonComponentView': defaultButtonComponentView,
      });
    });

    describe('Drawer Close Button', () => {
      const TRIGGER_TEXT = 'Open drawer';
      const MAIN_CONTEXT_TEXT = 'Main content goes here';

      const Component = ({ onCloseClick }: { onCloseClick: DrawerContentProps['onCloseClick'] }) => (
        <DesignSystemProvider>
          <Drawer.Root open>
            <Drawer.Trigger>{TRIGGER_TEXT}</Drawer.Trigger>
            <Drawer.Content title="drawer_title" componentId="drawer_test" onCloseClick={onCloseClick}>
              <div>{MAIN_CONTEXT_TEXT}</div>
            </Drawer.Content>
          </Drawer.Root>
        </DesignSystemProvider>
      );

      it('calls the onClick callback if provided', async () => {
        const onCloseClick = jest.fn();
        render(<Component onCloseClick={onCloseClick} />);

        const closeButton = screen.getByRole('button', { name: 'Close' });
        await userEvent.click(closeButton);

        expect(onCloseClick).toHaveBeenCalled();
      });
    });

    describe('Drawer Overlay', () => {
      it('Displays overlay only when the drawer is open', async () => {
        const OPEN_TEXT = 'Open button';

        function TestComponent() {
          const [open, setOpen] = useState(false);
          return (
            <DesignSystemProvider>
              <button onClick={() => setOpen(true)}> {OPEN_TEXT} </button>
              <Drawer.Root open={open}>
                <Drawer.Trigger>Open drawer</Drawer.Trigger>
                <Drawer.Content title="drawer_title" componentId="drawer_test">
                  <div>Main content goes here</div>
                </Drawer.Content>
              </Drawer.Root>
            </DesignSystemProvider>
          );
        }

        render(<TestComponent />);
        expect(screen.queryByTestId('drawer-overlay')).not.toBeInTheDocument();
        // Open the drawer
        await userEvent.click(screen.getByRole('button', { name: OPEN_TEXT }));

        // Now the overlay should be visible
        expect(screen.getByTestId('drawer-overlay')).toBeVisible();
      });

      it('Hides overlay when the drawer is nested', () => {
        render(
          <DesignSystemProvider>
            <Drawer.Root open>
              <Drawer.Content title="drawer_title" componentId="drawer_test">
                <Drawer.Root open>
                  <Drawer.Content title="nested_drawer_title" componentId="nested_drawer_test">
                    <Drawer.Root open>
                      <Drawer.Content title="deep_nested_drawer_title" componentId="deep_nested_drawer_test">
                        <div>Nested content</div>
                      </Drawer.Content>
                    </Drawer.Root>
                  </Drawer.Content>
                </Drawer.Root>
              </Drawer.Content>
            </Drawer.Root>
          </DesignSystemProvider>,
        );

        const drawerOverlays = screen.getAllByTestId('drawer-overlay');
        expect(drawerOverlays).toHaveLength(3);
        expect(drawerOverlays[0]).toBeVisible();
        expect(drawerOverlays[1]).not.toBeVisible();
        expect(drawerOverlays[2]).not.toBeVisible();
      });
    });
  },
);
