import { describe, it, expect } from '@jest/globals';
import { render, screen } from '@testing-library/react';
import userEvent from '@testing-library/user-event';
import { useRef, useEffect } from 'react';
import { useForm } from 'react-hook-form';

import type { InputRef } from '../../Input';
import { RHFControlledComponents } from '../RHFAdapters';
import { FormUI } from '../index';

const Fixture = ({
  initialValue = '',
  required = false,
  autoFocus = false,
}: {
  initialValue?: string;
  required?: boolean;
  autoFocus?: boolean;
}) => {
  const {
    control,
    watch,
    formState: { errors },
  } = useForm({
    mode: 'all',
    defaultValues: {
      data: {
        value: initialValue,
      },
    },
  });
  const value = watch('data.value');

  const inputRef = useRef<InputRef>(null);
  useEffect(() => {
    if (autoFocus) {
      inputRef.current?.focus();
    }
  }, [autoFocus]);

  return (
    <>
      <FormUI.Label htmlFor="test-input">Value</FormUI.Label>

      <RHFControlledComponents.Input
        id="test-input"
        componentId="test-rhf-input"
        control={control}
        name="data.value"
        rules={{ required }}
        inputRef={inputRef}
      />
      {errors?.data?.value?.type === 'required' && <FormUI.Message type="error" message="Missing required value" />}
      <div data-testid="actual-value">{value}</div>
    </>
  );
};

describe('RHF Adapter Input', () => {
  it('initializes from initial value', async () => {
    render(<Fixture initialValue="initial value" />);

    expect(await screen.findByLabelText('Value')).toHaveValue('initial value');
  });

  it('updates value correctly', async () => {
    render(<Fixture />);

    await userEvent.type(await screen.findByLabelText('Value'), 'new value');

    expect(await screen.findByLabelText('Value')).toHaveValue('new value');
  });

  it('applies validation correctly', async () => {
    render(<Fixture required={true} initialValue="value" />);

    await userEvent.clear(await screen.findByLabelText('Value'));

    expect(await screen.findByText('Missing required value')).toBeInTheDocument();
  });

  it('passes inputRef correctly', async () => {
    render(<Fixture autoFocus={true} />);

    expect(await screen.findByLabelText('Value')).toHaveFocus();
  });
});
