import {
  useCombobox,
  useMultipleSelection,
  type UseComboboxReturnValue,
  type UseComboboxStateChange,
  type UseMultipleSelectionReturnValue,
} from 'downshift';
import { useCallback, useEffect, useMemo, useRef, useState } from 'react';

import {
  DesignSystemEventProviderAnalyticsEventTypes,
  DesignSystemEventProviderComponentTypes,
  useDesignSystemEventComponentCallbacks,
} from '../../DesignSystemEventProvider';
import type { AnalyticsEventValueChangeNoPiiFlagProps } from '../../types';
import { safex } from '../../utils/safex';

interface SingleSelectProps<T> extends CommonComboboxStateProps<T> {
  multiSelect?: false;
  /* The state setter for inputValue */
  setInputValue?: React.Dispatch<React.SetStateAction<string>>;
  /* The state setter for items */
  setItems: React.Dispatch<React.SetStateAction<T[]>>;
}

interface MultiSelectProps<T> extends CommonComboboxStateProps<T> {
  multiSelect: true;
  /* The state setter for inputValue */
  setInputValue: React.Dispatch<React.SetStateAction<string>>;
  /* The list of currently selected items */
  selectedItems: T[];
  /* The state setter for selectedItems */
  setSelectedItems: React.Dispatch<React.SetStateAction<T[]>>;
}
interface CommonComboboxStateProps<T>
  extends AnalyticsEventValueChangeNoPiiFlagProps<DesignSystemEventProviderAnalyticsEventTypes.OnValueChange> {
  /**
   * The complete list of items, used for filtering. If you are going to render groups,
   * this list MUST be sorted so that groups are together and in the same order they are
   * rendered in the menu.
   */
  allItems: T[];
  /* The current list of items to render */
  items: T[];
  /* A function that returns whether the item matches the search query */
  matcher?: (item: T, searchQuery: string) => boolean;
  /* A function that returns the string representation of the item. If this arg is not supplied, toString() will be used */
  itemToString?: (item: T) => string;
  /* If using virtualization, this prop should be supplied. */
  onIsOpenChange?: (changes: UseComboboxStateChange<T>) => void;
  /* Whether to allow the input value to not match any options in the list. For this to work, the item type (T) must be a string.
   * Defaults to false.
   */
  allowNewValue?: boolean;
  /* For form libraries like RHF and AntD Form */
  formValue?: T;
  formOnChange?: (value: any) => void;
  formOnBlur?: (value: any) => void;
  onStateChange?: (changes: UseComboboxStateChange<T>) => void;
  /* Sets the initial selected item in the dropdown */
  initialSelectedItem?: T;
  /* Sets the initial input value. Use this prop to set the label for the initial selected item when the component mounts. */
  initialInputValue?: string;
  /* A temporary prop that will be removed after databricks.fe.designsystem.useComboboxStateUpdatesItemsWithEffect ramps:
   * Whether setItems should be called on allItems changes. This is helpful for not seeing unexpected items when allItems updates when
   * there is already an input value in the combobox */
  shouldSetItemsOnAllItemsChanges?: boolean;
  /**
   * When using RHF adapters, single select TypeaheadCombobox clears the value on blur, this prop prevents that behavior.
   */
  preventUnsetOnBlur?: boolean;
}

const mapItemsToString = <T>(items: T[], itemToString: (item: T | null) => string): string => {
  return JSON.stringify(items.map(itemToString));
};

export const TypeaheadComboboxStateChangeTypes = useCombobox.stateChangeTypes;
export const TypeaheadComboboxMultiSelectStateChangeTypes = useMultipleSelection.stateChangeTypes;

export type UseComboboxStateProps<T> = SingleSelectProps<T> | MultiSelectProps<T>;
export type ComboboxStateAnalyticsReturnValue<T> = UseComboboxReturnValue<T> &
  AnalyticsEventValueChangeNoPiiFlagProps<DesignSystemEventProviderAnalyticsEventTypes.OnValueChange>;

export function useComboboxState<T>({
  allItems,
  items,
  itemToString,
  onIsOpenChange,
  allowNewValue = false,
  formValue,
  formOnChange,
  formOnBlur,
  componentId,
  valueHasNoPii,
  analyticsEvents = [DesignSystemEventProviderAnalyticsEventTypes.OnValueChange],
  matcher,
  shouldSetItemsOnAllItemsChanges = safex('databricks.fe.designsystem.useComboboxStateUpdatesItemsWithEffect', false),
  preventUnsetOnBlur = false,
  ...props
}: UseComboboxStateProps<T>): ComboboxStateAnalyticsReturnValue<T> {
  const getFilteredItems = useCallback(
    (inputValue: string | undefined) => {
      const lowerCasedInputValue = inputValue?.toLowerCase() ?? '';
      // If the input is empty or if there is no matcher supplied, do not filter
      return allItems.filter((item: T) => !inputValue || !matcher || matcher(item, lowerCasedInputValue));
    },
    [allItems, matcher],
  );

  const [inputValue, setInputValue] = useState<string | undefined>();
  const memoizedAnalyticsEvents = useMemo(() => analyticsEvents, [analyticsEvents]);
  const eventContext = useDesignSystemEventComponentCallbacks({
    componentType: DesignSystemEventProviderComponentTypes.TypeaheadCombobox,
    componentId,
    analyticsEvents: memoizedAnalyticsEvents,
    valueHasNoPii,
  });

  const itemToStringWithDefaultToString = useCallback(
    (item: T | null) => {
      return item ? (itemToString ? itemToString(item) : item.toString()) : '';
    },
    [itemToString],
  );

  const prevAllItems = useRef(allItems);
  useEffect(() => {
    // When allItems changes, re-apply filter so users don't see stale items values in the dropdown box
    if (
      shouldSetItemsOnAllItemsChanges &&
      !props.multiSelect &&
      (allItems.length !== prevAllItems.current.length ||
        // Avoid redundant or endless updates by checking individual elements as allItems may have an unstable reference.
        allItems.some(
          (item, index) =>
            itemToStringWithDefaultToString(item) !== itemToStringWithDefaultToString(prevAllItems.current[index]),
        ))
    ) {
      props.setItems(getFilteredItems(inputValue));
      prevAllItems.current = allItems;
    }
  }, [allItems, inputValue, getFilteredItems, props, itemToStringWithDefaultToString, shouldSetItemsOnAllItemsChanges]);

  const comboboxState = {
    ...useCombobox({
      onIsOpenChange: onIsOpenChange,
      onInputValueChange: ({ inputValue }) => {
        if (inputValue !== undefined) {
          setInputValue(inputValue);
          props.setInputValue?.(inputValue);
        }
        if (!props.multiSelect) {
          props.setItems(getFilteredItems(inputValue));
        }
      },
      items: items,
      itemToString: itemToStringWithDefaultToString,
      defaultHighlightedIndex: props.multiSelect ? 0 : undefined, // after selection for multiselect, highlight the first item.
      scrollIntoView: () => {}, // disabling scroll because floating-ui repositions the menu
      selectedItem: props.multiSelect ? null : formValue, // useMultipleSelection will handle the item selection for multiselect
      stateReducer(state, actionAndChanges) {
        const { changes, type } = actionAndChanges;

        switch (type) {
          case useCombobox.stateChangeTypes.InputBlur:
            if (preventUnsetOnBlur) {
              return changes;
            }

            if (!props.multiSelect) {
              // If allowNewValue is true, register the input's current value on blur
              if (allowNewValue) {
                const newInputValue = state.inputValue === '' ? null : state.inputValue;
                formOnChange?.(newInputValue);
                formOnBlur?.(newInputValue);
              } else {
                // If allowNewValue is false, clear value on blur
                formOnChange?.(null);
                formOnBlur?.(null);
              }
            } else {
              formOnBlur?.(state.selectedItem);
            }
            return changes;
          case useCombobox.stateChangeTypes.InputKeyDownEnter:
          case useCombobox.stateChangeTypes.ItemClick:
            formOnChange?.(changes.selectedItem);
            return {
              ...changes,
              highlightedIndex: props.multiSelect ? state.highlightedIndex : 0, // on multiselect keep the highlighted index unchanged.
              isOpen: props.multiSelect ? true : false, // for multiselect, keep the menu open after selection.
            };
          default:
            return changes;
        }
      },
      onStateChange: (args) => {
        const { type, selectedItem: newSelectedItem, inputValue: newInputValue } = args;

        props.onStateChange?.(args);

        if (props.multiSelect) {
          switch (type) {
            case useCombobox.stateChangeTypes.InputKeyDownEnter:
            case useCombobox.stateChangeTypes.ItemClick:
            case useCombobox.stateChangeTypes.InputBlur:
              if (newSelectedItem) {
                props.setSelectedItems([...props.selectedItems, newSelectedItem]);
                props.setInputValue('');
                formOnBlur?.([...props.selectedItems, newSelectedItem]);
              }
              break;
            case useCombobox.stateChangeTypes.InputChange:
              props.setInputValue(newInputValue ?? '');
              break;
            case useCombobox.stateChangeTypes.FunctionReset:
              eventContext.onValueChange?.('[]');
              break;
            default:
              break;
          }
          // Unselect when clicking selected item
          if (newSelectedItem && props.selectedItems.includes(newSelectedItem)) {
            const newSelectedItems = props.selectedItems.filter((item) => item !== newSelectedItem);
            props.setSelectedItems(newSelectedItems);
            eventContext.onValueChange?.(mapItemsToString(newSelectedItems, itemToStringWithDefaultToString));
          } else if (newSelectedItem) {
            eventContext.onValueChange?.(
              mapItemsToString([...props.selectedItems, newSelectedItem], itemToStringWithDefaultToString),
            );
          }
        } else if (newSelectedItem) {
          eventContext.onValueChange?.(itemToStringWithDefaultToString(newSelectedItem));
        } else if (type === useCombobox.stateChangeTypes.FunctionReset) {
          eventContext.onValueChange?.('');
        }
      },
      initialInputValue: props.initialInputValue,
      initialSelectedItem: props.initialSelectedItem,
    }),
    componentId,
    analyticsEvents,
    valueHasNoPii,
  };

  return comboboxState;
}

interface AnalyticsConfig
  extends AnalyticsEventValueChangeNoPiiFlagProps<DesignSystemEventProviderAnalyticsEventTypes.OnValueChange> {
  itemToString?: (item: any) => string;
}

export function useMultipleSelectionState<T>(
  selectedItems: T[],
  setSelectedItems: React.Dispatch<React.SetStateAction<T[]>>,
  {
    componentId,
    analyticsEvents = [DesignSystemEventProviderAnalyticsEventTypes.OnValueChange],
    valueHasNoPii,
    itemToString,
  }: AnalyticsConfig,
): UseMultipleSelectionReturnValue<T> {
  const memoizedAnalyticsEvents = useMemo(() => analyticsEvents, [analyticsEvents]);
  const eventContext = useDesignSystemEventComponentCallbacks({
    componentType: DesignSystemEventProviderComponentTypes.TypeaheadCombobox,
    componentId,
    analyticsEvents: memoizedAnalyticsEvents,
    valueHasNoPii,
  });

  return useMultipleSelection({
    selectedItems,
    onStateChange({ selectedItems: newSelectedItems, type }) {
      switch (type) {
        case useMultipleSelection.stateChangeTypes.SelectedItemKeyDownBackspace:
        case useMultipleSelection.stateChangeTypes.SelectedItemKeyDownDelete:
        case useMultipleSelection.stateChangeTypes.DropdownKeyDownBackspace:
        case useMultipleSelection.stateChangeTypes.FunctionRemoveSelectedItem:
          setSelectedItems(newSelectedItems || []);
          break;
        default:
          break;
      }
      const itemToStringWithDefaultToString = itemToString ?? ((item) => item?.toString() ?? '');
      eventContext.onValueChange?.(mapItemsToString(newSelectedItems || [], itemToStringWithDefaultToString));
    },
  });
}
