import { describe, it, jest, expect } from '@jest/globals';
import { render, screen } from '@testing-library/react';
import userEvent from '@testing-library/user-event';
import type { UseFormHandleSubmit } from 'react-hook-form';

import { Form, RhfForm } from './Form';
import { Button, DesignSystemEventProvider } from '../../design-system';

describe('Form', () => {
  it('renders the form and handles submission', async () => {
    // Arrange
    const handleSubmit = jest.fn<() => Promise<void>>().mockResolvedValue(undefined);
    render(
      <Form componentId="testForm" onSubmit={handleSubmit}>
        <button type="submit">Submit</button>
      </Form>,
    );

    // Act
    await userEvent.click(screen.getByRole('button', { name: /submit/i }));

    // Assert
    expect(handleSubmit).toHaveBeenCalledTimes(1);
  });

  it('calls the DesignSystemEventProvider onSubmit callback', async () => {
    // Arrange
    const handleSubmit = jest.fn<() => Promise<void>>().mockResolvedValue(undefined);
    const eventCallback = jest.fn();
    render(
      <DesignSystemEventProvider callback={eventCallback}>
        <Form componentId="eventForm" onSubmit={handleSubmit}>
          <Button componentId="submit-button-test" htmlType="submit">
            Submit
          </Button>
        </Form>
      </DesignSystemEventProvider>,
    );

    // Act
    await userEvent.click(screen.getByRole('button', { name: /submit/i }));

    // Assert
    expect(handleSubmit).toHaveBeenCalledTimes(1);
    expect(eventCallback).toHaveBeenNthCalledWith(2, {
      eventType: 'onSubmit',
      componentId: 'eventForm',
      componentType: 'form',
      componentSubType: undefined,
      shouldStartInteraction: true,
      value: undefined,
      mode: 'default',
      formPropertyValues: {
        initial: undefined,
        final: undefined,
      },
      referrerComponent: {
        id: 'submit-button-test',
        type: 'button',
      },
      event: expect.anything(),
    });
  });

  it('throws error if nested form components are rendered', () => {
    // Arrange
    const handleSubmit = jest.fn<() => Promise<void>>().mockResolvedValue(undefined);

    // Assert
    expect(() => {
      render(
        <Form componentId="outerForm" onSubmit={handleSubmit}>
          <Form componentId="innerForm" onSubmit={handleSubmit}>
            <button type="submit">Submit</button>
          </Form>
        </Form>,
      );
    }).toThrowError('DuBois Form component cannot be nested');
  });

  it('calls the DesignSystemEventProvider onSubmit callback with initial & final values', async () => {
    // Arrange
    const handleSubmit: UseFormHandleSubmit<any> = (onValid, _) => async (e) => {
      await onValid({ testProperty: 123 }, e);
    };
    const eventCallback = jest.fn();
    const onValidCallback = jest.fn();
    render(
      <DesignSystemEventProvider callback={eventCallback}>
        <RhfForm
          componentId="eventForm"
          handleSubmit={handleSubmit}
          handleValid={onValidCallback}
          initialState={{ testProperty: 321, testFooProperty: false, testBarProperty: 'test' }}
        >
          <Button componentId="submit-button-test" htmlType="submit">
            Submit
          </Button>
        </RhfForm>
      </DesignSystemEventProvider>,
    );

    // Act
    await userEvent.click(screen.getByRole('button', { name: /submit/i }));

    // Assert
    expect(onValidCallback).toHaveBeenCalledTimes(1);
    expect(eventCallback).toHaveBeenCalledTimes(3);
    expect(eventCallback).toHaveBeenNthCalledWith(1, {
      eventType: 'onClick',
      componentId: 'submit-button-test',
      componentType: 'button',
      componentSubType: undefined,
      shouldStartInteraction: true,
      value: undefined,
      event: expect.anything(),
      isInteractionSubject: false,
    });
    expect(eventCallback).toHaveBeenNthCalledWith(2, {
      eventType: 'onSubmit',
      componentId: 'eventForm',
      componentType: 'form',
      componentSubType: undefined,
      shouldStartInteraction: true,
      value: undefined,
      mode: 'associate_event_only',
      formPropertyValues: {
        initial: undefined,
        final: undefined,
      },
      referrerComponent: {
        id: 'submit-button-test',
        type: 'button',
      },
      event: expect.anything(),
    });
    expect(eventCallback).toHaveBeenNthCalledWith(3, {
      eventType: 'onSubmit',
      componentId: 'eventForm',
      componentType: 'form',
      componentSubType: undefined,
      shouldStartInteraction: true,
      value: undefined,
      mode: 'default',
      formPropertyValues: {
        initial: {
          testProperty: 321,
          testFooProperty: false,
          testBarProperty: 'test',
        },
        final: {
          testProperty: 123,
        },
      },
      referrerComponent: {
        id: 'submit-button-test',
        type: 'button',
      },
      event: expect.anything(),
    });
  });
});
