import type { FormEvent } from 'react';
import React, { useCallback, useMemo, useRef, useState } from 'react';
import type { DeepPartial, FieldValues, UseFormHandleSubmit, SubmitHandler, SubmitErrorHandler } from 'react-hook-form';

import type { ReferrerComponentType } from '../../design-system';
import {
  DesignSystemEventProviderAnalyticsEventTypes,
  DesignSystemEventProviderComponentTypes,
  useDesignSystemEventComponentCallbacks,
} from '../../design-system/DesignSystemEventProvider';
import { LoadingState } from '../../design-system/LoadingState';
import type { AnalyticsEventPropsWithStartInteraction } from '../../design-system/types';

const FormContext = React.createContext<{
  componentId?: string;
  isSubmitting: boolean;
  formRef?: React.MutableRefObject<HTMLFormElement | null>;
}>({ componentId: undefined, isSubmitting: false, formRef: undefined });

export const useFormContext = () => React.useContext(FormContext);

export type FormProps = {
  onSubmit: (event: FormEvent) => Promise<void>;
} & AnalyticsEventPropsWithStartInteraction<DesignSystemEventProviderAnalyticsEventTypes.OnSubmit> &
  Omit<React.ComponentProps<'form'>, 'onSubmit'>;

/**
 * Form is a wrapper around the form element that allows us to track the active element before the form is submitted.
 * This is useful for analytics purposes to know what component was last focused when the form was sent for submission.
 *
 * NOTE: Form component cannot be nested.
 */
export const Form = ({
  children,
  componentId,
  analyticsEvents = [DesignSystemEventProviderAnalyticsEventTypes.OnSubmit],
  onSubmit,
  shouldStartInteraction,
  ...otherProps
}: FormProps) => {
  const formContext = useFormContext();
  const formRef = useRef<HTMLFormElement | null>(null);
  const eventRef = useRef<React.UIEvent | null>(null);
  if (formContext.componentId !== undefined) {
    throw new Error('DuBois Form component cannot be nested');
  }

  const memoizedAnalyticsEvents = useMemo(() => analyticsEvents, [analyticsEvents]);
  const eventContext = useDesignSystemEventComponentCallbacks({
    componentType: DesignSystemEventProviderComponentTypes.Form,
    componentId,
    analyticsEvents: memoizedAnalyticsEvents,
    shouldStartInteraction,
  });

  const [isSubmitting, setIsSubmitting] = useState(false);

  const onSubmitHandler = useCallback<React.FormEventHandler<HTMLFormElement>>(
    async (e) => {
      try {
        setIsSubmitting(true);
        eventContext.onSubmit({
          event: e,
          initialState: undefined,
          finalState: undefined,
          referrerComponent: extractReferrerComponent(),
        });
        await onSubmit(e);
      } finally {
        setIsSubmitting(false);
      }
    },
    [eventContext, onSubmit],
  );
  const contextValue = useMemo(() => ({ componentId, isSubmitting, eventRef, formRef }), [componentId, isSubmitting]);

  return (
    <FormContext.Provider value={contextValue}>
      <form onSubmit={onSubmitHandler} {...otherProps} ref={formRef} {...eventContext.dataComponentProps}>
        {isSubmitting && <LoadingState description={componentId} />}
        {children}
      </form>
    </FormContext.Provider>
  );
};

export type RhfFormProps<T extends FieldValues> = {
  /**
   * @param handleSubmit - The handleSubmit function from react-hook-form
   */
  handleSubmit: UseFormHandleSubmit<T>;
  /**
   * @param handleValid - The onValid callback for handleSubmit from react-hook-form; this is called when the form is
   * valid according to react-hook-form and after the form is submitted
   */
  handleValid: SubmitHandler<T>;
  /**
   * @param handleInvalid - The onInvalid callback for handleSubmit from react-hook-form; this is called when the form is
   * invalid according to react-hook-form and after the form is submitted
   */
  handleInvalid?: SubmitErrorHandler<T>;
  /**
   * @param initialState - The values of the form when it was first rendered with its default values
   */
  initialState?: DeepPartial<T>;
  /**
   * @param processFinalPropertyValues - A function that can be used to process the data that is passed to the
   * final_values property in the FORM_SUBMIT event
   */
  processFinalPropertyValues?: (data: T) => T;
} & AnalyticsEventPropsWithStartInteraction<DesignSystemEventProviderAnalyticsEventTypes.OnSubmit> &
  Omit<React.ComponentProps<'form'>, 'onSubmit'>;

/**
 * Form is a wrapper around the Form DuBois component to better integrate with React Hook Form and to track the active
 * element before the form is submitted. This is useful for analytics purposes to know what component was last focused
 * when the form was sent for submission, along with being able to pass in the initial and final values of the form.
 *
 * Internal implementation details:
 * The form is wrapped in a Form component to share implementation details along with associating the HTML form
 * submission event with interaction details. The Form component does not emit analytics events, but the RHF
 * component will emit the events once the RHF lifecycle events occur so we can relay the form property values.
 *
 * NOTE: Form components cannot be nested.
 */
export const RhfForm = <T extends FieldValues>({
  handleSubmit,
  handleValid,
  handleInvalid,
  processFinalPropertyValues,
  children,
  componentId,
  analyticsEvents = [DesignSystemEventProviderAnalyticsEventTypes.OnSubmit],
  shouldStartInteraction,
  initialState,
  ...otherProps
}: RhfFormProps<T>) => {
  const memoizedAnalyticsEvents = useMemo(() => analyticsEvents, [analyticsEvents]);
  const eventContext = useDesignSystemEventComponentCallbacks({
    componentType: DesignSystemEventProviderComponentTypes.Form,
    componentId,
    analyticsEvents: memoizedAnalyticsEvents,
    shouldStartInteraction,
  });
  const onValidWrapper: SubmitHandler<T> = useCallback(
    async (data, e) => {
      const refinedData = processFinalPropertyValues ? processFinalPropertyValues(data) : data;
      eventContext.onSubmit({
        event: e as FormEvent,
        initialState,
        finalState: refinedData,
        referrerComponent: extractReferrerComponent(),
      });
      await handleValid(refinedData, e);
    },
    [eventContext, initialState, handleValid, processFinalPropertyValues],
  );
  const onInvalidWrapper: SubmitErrorHandler<T> = useCallback(
    async (errors, e) => {
      eventContext.onSubmit({
        event: e as FormEvent,
        initialState,
        finalState: undefined,
        referrerComponent: extractReferrerComponent(),
      });
      if (handleInvalid) {
        await handleInvalid(errors, e);
      }
    },
    [eventContext, initialState, handleInvalid],
  );
  const onSubmitWrapper = useCallback(
    async (e) => {
      await handleSubmit(onValidWrapper, onInvalidWrapper)(e);
    },
    [handleSubmit, onInvalidWrapper, onValidWrapper],
  );
  return (
    <Form
      componentId={componentId}
      analyticsEvents={[]} // Needed to disable events
      onSubmit={onSubmitWrapper}
      {...otherProps}
    >
      {children}
    </Form>
  );
};

function extractReferrerComponent(): ReferrerComponentType | undefined {
  const activeElement = document.activeElement as HTMLOrSVGElement | null;
  const { componentType, componentId } = activeElement?.dataset ?? {};
  if (componentType && componentId) {
    return {
      type: componentType as DesignSystemEventProviderComponentTypes,
      id: componentId,
    };
  }

  return undefined;
}
