/** @jsxImportSource @emotion/react */
import { css } from '@emotion/react';
import React, { useCallback, useMemo, useState, useRef } from 'react';

import { ListboxInput } from './ListboxInput';
import { ListboxOptions } from './ListboxOptions';
import { ListboxRoot, useListboxContext } from './ListboxRoot';
import type { ListboxProps } from './types';
import { useDesignSystemTheme } from '../../design-system';

const ListboxContent = ({
  options,
  filterValue,
  setFilterValue,
  filterInputPlaceholder,
  onSelect,
  ariaLabel,
  includeFilterInput,
  filterInputEmptyMessage,
  listBoxDivRef,
}: {
  options: ListboxProps['options'];
  filterValue: string;
  setFilterValue: (value: string) => void;
  filterInputPlaceholder?: string;
  onSelect: (value: string) => void;
  ariaLabel: string;
  includeFilterInput?: boolean;
  filterInputEmptyMessage?: string;
  listBoxDivRef?: React.RefObject<HTMLDivElement>;
}) => {
  const [highlightedValue, setHighlightedValue] = useState<string>();
  const { listboxId } = useListboxContext();
  const designSystemTheme = useDesignSystemTheme();

  const filteredOptions = useMemo(() => {
    if (!filterValue) return options;
    const lowerFilter = filterValue.toLowerCase();
    return options.filter(
      (option) => option.value.toLowerCase().includes(lowerFilter) || option.label.toLowerCase().includes(lowerFilter),
    );
  }, [filterValue, options]);

  return (
    <div
      css={css({
        display: 'flex',
        flexDirection: 'column',
        gap: '8px',
      })}
      ref={listBoxDivRef}
    >
      {includeFilterInput && (
        <>
          <ListboxInput
            value={filterValue}
            onChange={setFilterValue}
            placeholder={filterInputPlaceholder}
            aria-controls={listboxId}
            aria-activedescendant={highlightedValue ? `${listboxId}-${highlightedValue}` : undefined}
            options={filteredOptions}
          />
          {filteredOptions.length === 0 && filterInputEmptyMessage && (
            <div
              aria-live="polite"
              role="status"
              css={{
                color: designSystemTheme.theme.colors.textSecondary,
                textAlign: 'center',
                padding: '6px 12px',
                width: '100%',
                boxSizing: 'border-box',
              }}
            >
              {filterInputEmptyMessage}
            </div>
          )}
        </>
      )}
      <ListboxOptions
        options={filteredOptions}
        onSelect={onSelect}
        onHighlight={setHighlightedValue}
        aria-label={ariaLabel}
      />
    </div>
  );
};

export const Listbox = ({
  options,
  onSelect,
  includeFilterInput,
  filterInputEmptyMessage,
  initialSelectedValue,
  filterInputPlaceholder,
  'aria-label': ariaLabel,
  className,
}: ListboxProps) => {
  const [filterValue, setFilterValue] = useState('');

  const handleSelect = useCallback(
    (value: string) => {
      onSelect?.(value);
    },
    [onSelect],
  );

  const listBoxDivRef = useRef<HTMLDivElement>(null);

  return (
    <ListboxRoot
      className={className}
      onSelect={handleSelect}
      initialSelectedValue={initialSelectedValue}
      listBoxDivRef={listBoxDivRef}
    >
      <ListboxContent
        options={options}
        filterValue={filterValue}
        setFilterValue={setFilterValue}
        filterInputPlaceholder={filterInputPlaceholder}
        onSelect={handleSelect}
        ariaLabel={ariaLabel}
        includeFilterInput={includeFilterInput}
        filterInputEmptyMessage={filterInputEmptyMessage}
        listBoxDivRef={listBoxDivRef}
      />
    </ListboxRoot>
  );
};

export type { ListboxProps } from './types';
