import React, { useCallback, useRef, useMemo } from 'react';

import type {
  TreeGridRow,
  TreeGridRowId,
  TreeGridState,
  TreeGridColumn,
  TreeGridRenderCellArgs,
  TreeGridRenderRowArgs,
  TreeGridRenderTableArgs,
  TreeGridRenderHeaderArgs,
  TreeGridInitialState,
} from './types';
import { useDefaultTreeGridState } from './useDefaultTreeGridState';

interface TreeGridRowWithInternalMetadata extends TreeGridRow {
  /** The depth of the row */
  depth: number;
  /** The parent row's id */
  parentId: TreeGridRowId | null;
}

export interface TreeGridProps {
  /** The data to be displayed in the grid */
  data: TreeGridRow[];
  /** The columns to be displayed in the grid */
  columns: TreeGridColumn[];
  /** Function to render the cell content */
  renderCell: (args: TreeGridRenderCellArgs) => React.ReactElement;
  /** Optional function to render the row */
  renderRow?: (args: TreeGridRenderRowArgs) => React.ReactElement | null;
  /** Optional function to render the entire table */
  renderTable?: (args: TreeGridRenderTableArgs) => React.ReactElement;
  /** Optional function to render the header */
  renderHeader?: (args: TreeGridRenderHeaderArgs) => React.ReactElement;
  /** Callback function when a row is selected via the keyboard */
  onRowKeyboardSelect?: (rowId: string | number) => void;
  /** Callback function when a cell is selected via the keyboard */
  onCellKeyboardSelect?: (rowId: string | number, columnKey: string) => void;
  /** Whether to include a header in the grid */
  includeHeader?: boolean;
  /**
   * Configures state management for the TreeGrid.
   *
   * When no `state` is provided or when `initialState` is set, the state will be managed within
   * the TreeGrid. Otherwise, the given custom state implementation is used.
   */
  state?: { initialState: TreeGridInitialState } | TreeGridState;
}

const flattenData = (
  data: TreeGridRow[],
  expandedRows: Record<string | number, boolean>,
  depth = 0,
  parentId: string | number | null = null,
): TreeGridRowWithInternalMetadata[] => {
  return data.reduce((acc: TreeGridRowWithInternalMetadata[], node) => {
    acc.push({ ...node, depth, parentId });
    if (node.children && expandedRows[node.id]) {
      acc.push(...flattenData(node.children, expandedRows, depth + 1, node.id));
    }
    return acc;
  }, []);
};

const findFocusableElementForCellIndex = (row: HTMLTableRowElement, cellIndex: number): HTMLElement | null => {
  const cell = row.cells[cellIndex] as HTMLTableCellElement;
  return cell?.querySelector<HTMLElement>('[tabindex], button, a, input, select, textarea') || null;
};

const findNextFocusableCellIndexInRow = (
  row: HTMLTableRowElement,
  columns: TreeGridColumn[],
  startIndex: number,
  direction: 'next' | 'previous',
) => {
  const cells = Array.from(row.cells);
  const increment = direction === 'next' ? 1 : -1;
  const limit = direction === 'next' ? cells.length : -1;

  for (let i = startIndex + increment; i !== limit; i += increment) {
    const cell = cells[i];
    const column = columns[i];
    const focusableElement = findFocusableElementForCellIndex(row, i);
    const cellContent = cell?.textContent?.trim();

    if (focusableElement || (!column.contentFocusable && cellContent)) {
      return i;
    }
  }
  return -1;
};

export const TreeGrid: React.FC<TreeGridProps> = ({
  data,
  columns,
  renderCell,
  renderRow,
  renderTable,
  renderHeader,
  onRowKeyboardSelect,
  onCellKeyboardSelect,
  includeHeader = false,
  state: providedState,
}) => {
  const defaultState = useDefaultTreeGridState({
    initialState: providedState && 'initialState' in providedState ? providedState.initialState : undefined,
  });
  const { expandedRows, activeRowId, toggleRowExpanded, setActiveRowId } =
    providedState && !('initialState' in providedState) ? providedState : defaultState;

  const gridRef = useRef<HTMLTableElement>(null);

  const flattenedData = useMemo(() => flattenData(data, expandedRows), [data, expandedRows]);

  const focusRow = useCallback(
    ({ rowId, rowIndex }: { rowId: TreeGridRowId; rowIndex: number }) => {
      const row = gridRef.current?.querySelector(`tbody tr:nth-child(${rowIndex + 1})`) as HTMLTableRowElement;
      row?.focus();
      setActiveRowId(rowId);
    },
    [setActiveRowId],
  );

  const focusElement = useCallback(
    (element: HTMLElement | null, rowIndex: number) => {
      if (element) {
        element.focus();
        setActiveRowId(flattenedData[rowIndex].id);
      }
    },
    [setActiveRowId, flattenedData],
  );

  const handleKeyDown = useCallback(
    (event: React.KeyboardEvent<HTMLTableRowElement>, rowIndex: number) => {
      const { key } = event;
      let newRowIndex = rowIndex;
      const closestTd = (event.target as HTMLElement).closest('td') as HTMLTableCellElement;

      if (!gridRef.current || !gridRef.current.contains(document.activeElement)) {
        return;
      }

      const handleArrowVerticalNavigation = (direction: 'next' | 'previous') => {
        if (closestTd) {
          const currentCellIndex = closestTd.cellIndex;
          let targetRow = closestTd.closest('tr')?.[`${direction}ElementSibling`] as HTMLTableRowElement;

          const moveFocusToRow = (row: HTMLTableRowElement) => {
            const focusableElement = findFocusableElementForCellIndex(row, currentCellIndex);
            const cellContent = row.cells[currentCellIndex]?.textContent?.trim();
            if (focusableElement || (!columns[currentCellIndex].contentFocusable && cellContent)) {
              event.preventDefault();
              focusElement(
                focusableElement || row.cells[currentCellIndex],
                flattenedData.findIndex((r) => r.id === row.dataset['id']),
              );
              return true;
            }
            return false;
          };

          while (targetRow) {
            if (moveFocusToRow(targetRow)) return;
            targetRow = targetRow[`${direction}ElementSibling`] as HTMLTableRowElement;
          }
        } else if (document.activeElement instanceof HTMLTableRowElement) {
          if (direction === 'next') {
            newRowIndex = Math.min(rowIndex + 1, flattenedData.length - 1);
          } else {
            newRowIndex = Math.max(rowIndex - 1, 0);
          }
        }
      };

      const handleArrowHorizontalNavigation = (direction: 'next' | 'previous') => {
        if (closestTd) {
          const currentRow = closestTd.closest('tr') as HTMLTableRowElement;
          let targetCellIndex = closestTd.cellIndex;

          targetCellIndex = findNextFocusableCellIndexInRow(currentRow, columns, targetCellIndex, direction);

          if (targetCellIndex !== -1) {
            event.preventDefault();
            const targetCell = currentRow.cells[targetCellIndex];
            const focusableElement = findFocusableElementForCellIndex(currentRow, targetCellIndex);
            focusElement(focusableElement || targetCell, rowIndex);
            return;
          } else if (direction === 'previous' && targetCellIndex === -1) {
            // If we're at the leftmost cell, focus on the row
            event.preventDefault();
            currentRow.focus();
            return;
          }
        }

        if (document.activeElement instanceof HTMLTableRowElement) {
          const currentRow = document.activeElement as HTMLTableRowElement;
          if (direction === 'next') {
            if (flattenedData[rowIndex].children) {
              if (!expandedRows[flattenedData[rowIndex].id]) {
                toggleRowExpanded(flattenedData[rowIndex].id);
              } else {
                const firstCell = currentRow.cells[0];
                focusElement(firstCell, rowIndex);
              }
            } else {
              const firstFocusableCell = findNextFocusableCellIndexInRow(currentRow, columns, -1, 'next');
              if (firstFocusableCell !== -1) {
                focusElement(currentRow.cells[firstFocusableCell], rowIndex);
              }
            }
          } else {
            if (expandedRows[flattenedData[rowIndex].id]) {
              toggleRowExpanded(flattenedData[rowIndex].id);
            } else if (flattenedData[rowIndex].depth && flattenedData[rowIndex].depth > 0) {
              newRowIndex = flattenedData.findIndex((row) => row.id === flattenedData[rowIndex].parentId);
            }
          }
          return;
        }

        // If we're at the edge of the row, handle expanding/collapsing or moving to parent/child
        if (direction === 'next') {
          if (flattenedData[rowIndex].children && !expandedRows[flattenedData[rowIndex].id]) {
            toggleRowExpanded(flattenedData[rowIndex].id);
          }
        } else {
          if (expandedRows[flattenedData[rowIndex].id]) {
            toggleRowExpanded(flattenedData[rowIndex].id);
          } else if (flattenedData[rowIndex].depth && flattenedData[rowIndex].depth > 0) {
            newRowIndex = flattenedData.findIndex((row) => row.id === flattenedData[rowIndex].parentId);
          }
        }
      };

      const handleEnterKey = () => {
        if (closestTd) {
          onCellKeyboardSelect?.(flattenedData[rowIndex].id, columns[closestTd.cellIndex].id);
        } else if (document.activeElement instanceof HTMLTableRowElement) {
          onRowKeyboardSelect?.(flattenedData[rowIndex].id);
        }
      };

      switch (key) {
        case 'ArrowUp':
          handleArrowVerticalNavigation('previous');
          break;
        case 'ArrowDown':
          handleArrowVerticalNavigation('next');
          break;
        case 'ArrowLeft':
          handleArrowHorizontalNavigation('previous');
          break;
        case 'ArrowRight':
          handleArrowHorizontalNavigation('next');
          break;
        case 'Enter':
          handleEnterKey();
          break;
        default:
          return;
      }

      if (newRowIndex !== rowIndex) {
        event.preventDefault();
        focusRow({ rowId: flattenedData[newRowIndex].id, rowIndex: newRowIndex });
      }
    },
    [
      expandedRows,
      columns,
      flattenedData,
      toggleRowExpanded,
      onRowKeyboardSelect,
      onCellKeyboardSelect,
      focusRow,
      focusElement,
    ],
  );

  const defaultRenderRow = useCallback(
    ({ rowProps, children }: TreeGridRenderRowArgs) => <tr {...rowProps}>{children}</tr>,
    [],
  );

  const defaultRenderTable = useCallback(
    ({ tableProps, children }: TreeGridRenderTableArgs) => <table {...tableProps}>{children}</table>,
    [],
  );

  const defaultRenderHeader = useCallback(
    ({ columns, headerProps }: TreeGridRenderHeaderArgs) => (
      <thead {...headerProps}>
        <tr>
          {columns.map((column) => (
            <th key={column.id} role="columnheader">
              {column.header}
            </th>
          ))}
        </tr>
      </thead>
    ),
    [],
  );

  const renderRowWrapper = useCallback(
    (row: TreeGridRowWithInternalMetadata, rowIndex: number) => {
      const isExpanded = expandedRows[row.id];
      const isKeyboardActive = row.id === (activeRowId ?? flattenedData[0].id);

      const rowProps: React.HTMLAttributes<HTMLTableRowElement> & { key: React.Key; [key: `data-${string}`]: any } = {
        key: row.id,
        'data-id': row.id,
        role: 'row',
        'aria-selected': false,
        'aria-level': (row.depth || 0) + 1,
        'aria-expanded': row.children ? (isExpanded ? 'true' : 'false') : undefined,
        tabIndex: isKeyboardActive ? 0 : -1,
        onKeyDown: (e) => handleKeyDown(e, rowIndex),
      };

      const children = columns.map((column, colIndex) => {
        const cellProps: React.TdHTMLAttributes<HTMLTableCellElement> & { key: string } = {
          key: `${row.id}-${column.id}`,
          role: column.isRowHeader ? 'rowheader' : 'gridcell',
          tabIndex: column.contentFocusable ? undefined : isKeyboardActive ? 0 : -1,
        };

        return renderCell({
          row,
          column,
          rowDepth: row.depth || 0,
          rowIndex,
          colIndex,
          rowIsKeyboardActive: isKeyboardActive,
          rowIsExpanded: isExpanded,
          toggleRowExpanded,
          cellProps,
        });
      });

      return (renderRow || defaultRenderRow)({
        row,
        rowIndex,
        isExpanded,
        isKeyboardActive,
        rowProps,
        children,
      });
    },
    [
      activeRowId,
      flattenedData,
      expandedRows,
      handleKeyDown,
      renderCell,
      renderRow,
      defaultRenderRow,
      toggleRowExpanded,
      columns,
    ],
  );

  return (renderTable || defaultRenderTable)({
    tableProps: { role: 'treegrid', ref: gridRef },
    children: (
      <>
        {includeHeader && (renderHeader || defaultRenderHeader)({ columns, headerProps: {} })}
        <tbody>{flattenedData.map((row, index) => renderRowWrapper(row, index))}</tbody>
      </>
    ),
  });
};
